@@ -4,7 +4,7 @@
 
 package Convert::ASN1;
 {
-  $Convert::ASN1::VERSION = '0.26';
+  $Convert::ASN1::VERSION = '0.27';
 }
 
 ##
@@ -84,6 +84,7 @@ sub asn_dump {
     
     my $start = $pos;
     my($tb,$tag,$tnum) = asn_decode_tag2(substr($_[0],$pos,10));
+    last unless defined $tb;
     $pos += $tb;
     my($lb,$len) = asn_decode_length(substr($_[0],$pos,10));
     $pos += $lb;
@@ -160,6 +161,7 @@ sub asn_dump {
     }
     $pos += $len;
   }
+  printf "Buffer contains %d extra bytes\n", $length - $pos if $pos < $length;
 
   select($ofh);
 }
