@@ -4,7 +4,7 @@
 
 package Convert::ASN1;
 {
-  $Convert::ASN1::VERSION = '0.25';
+  $Convert::ASN1::VERSION = '0.27';
 }
 
 use strict;
@@ -83,7 +83,7 @@ sub _decode {
 	    next OP;
 	  }
 
-	  if ($tag eq ($op->[cTAG] | chr(ASN_CONSTRUCTOR))
+	  if ($tag eq ($op->[cTAG] | pack("C",ASN_CONSTRUCTOR))
 	      and my $ctr = $ctr[$op->[cTYPE]]) 
 	  {
 	    _decode(
@@ -210,7 +210,7 @@ sub _decode {
 		next OP;
 	      }
 
-	      if ($tag eq ($cop->[cTAG] | chr(ASN_CONSTRUCTOR))
+	      if ($tag eq ($cop->[cTAG] | pack("C",ASN_CONSTRUCTOR))
 		  and my $ctr = $ctr[$cop->[cTYPE]]) 
 	      {
 		my $nstash = $seqof
@@ -265,7 +265,7 @@ sub _dec_boolean {
 # 0      1    2       3     4     5     6
 # $optn, $op, $stash, $var, $buf, $pos, $len
 
-  $_[3] = ord(substr($_[4],$_[5],1)) ? 1 : 0;
+  $_[3] = unpack("C",substr($_[4],$_[5],1)) ? 1 : 0;
   1;
 }
 
@@ -275,9 +275,9 @@ sub _dec_integer {
 # $optn, $op, $stash, $var, $buf, $pos, $len
 
   my $buf = substr($_[4],$_[5],$_[6]);
-  my $tmp = ord($buf) & 0x80 ? chr(255) : chr(0);
+  my $tmp = unpack("C",$buf) & 0x80 ? pack("C",255) : pack("C",0);
   if ($_[6] > 4) {
-      $_[3] = os2ip($tmp x (4-$_[6]) . $buf, $_[0]->{decode_bigint} || 'Math::BigInt');
+      $_[3] = os2ip($buf, $_[0]->{decode_bigint} || 'Math::BigInt');
   } else {
       # N unpacks an unsigned value
       $_[3] = unpack("l",pack("l",unpack("N", $tmp x (4-$_[6]) . $buf)));
@@ -290,7 +290,7 @@ sub _dec_bitstring {
 # 0      1    2       3     4     5     6
 # $optn, $op, $stash, $var, $buf, $pos, $len
 
-  $_[3] = [ substr($_[4],$_[5]+1,$_[6]-1), ($_[6]-1)*8-ord(substr($_[4],$_[5],1)) ];
+  $_[3] = [ substr($_[4],$_[5]+1,$_[6]-1), ($_[6]-1)*8-unpack("C",substr($_[4],$_[5],1)) ];
   1;
 }
 
@@ -342,7 +342,7 @@ sub _dec_real {
 
   $_[3] = 0.0, return unless $_[6];
 
-  my $first = ord(substr($_[4],$_[5],1));
+  my $first = unpack("C",substr($_[4],$_[5],1));
   if ($first & 0x80) {
     # A real number
 
@@ -354,7 +354,7 @@ sub _dec_real {
 
     if($expLen == 3) {
       $estart++;
-      $expLen = ord(substr($_[4],$_[5]+1,1));
+      $expLen = unpack("C",substr($_[4],$_[5]+1,1));
     }
     else {
       $expLen++;
@@ -389,7 +389,7 @@ sub _dec_explicit {
 # 0      1    2       3     4     5     6     7
 # $optn, $op, $stash, $var, $buf, $pos, $len, $larr
 
-  local $_[1][cCHILD][0][cVAR] = $_[1][cVAR];
+  local $_[1][cCHILD][0][cVAR] = $_[1][cVAR] unless $_[1][cCHILD][0][cVAR];
 
   _decode(
     $_[0], #optn
@@ -465,7 +465,7 @@ SET_OP:
 	  $done = $idx;
 	  last SET_OP;
 	}
-	if ($tag eq ($op->[cTAG] | chr(ASN_CONSTRUCTOR))
+	if ($tag eq ($op->[cTAG] | pack("C",ASN_CONSTRUCTOR))
 	    and my $ctr = $ctr[$op->[cTYPE]]) 
 	{
 	  _decode(
@@ -505,7 +505,7 @@ SET_OP:
 	    $done = $idx;
 	    last SET_OP;
 	  }
-	  if ($tag eq ($cop->[cTAG] | chr(ASN_CONSTRUCTOR))
+	  if ($tag eq ($cop->[cTAG] | pack("C",ASN_CONSTRUCTOR))
 	      and my $ctr = $ctr[$cop->[cTYPE]]) 
 	  {
 	    my $nstash = defined($var) ? ($stash->{$var}={}) : $stash;
@@ -623,14 +623,17 @@ sub _dec_utf8 {
 sub _decode_tl {
   my($pos,$end,$larr) = @_[1,2,3];
 
+  return if $pos >= $end;
+
   my $indef = 0;
 
   my $tag = substr($_[0], $pos++, 1);
 
-  if((ord($tag) & 0x1f) == 0x1f) {
+  if((unpack("C",$tag) & 0x1f) == 0x1f) {
     my $b;
     my $n=1;
     do {
+      return if $pos >= $end;
       $tag .= substr($_[0],$pos++,1);
       $b = ord substr($tag,-1);
     } while($b & 0x80);
@@ -645,7 +648,8 @@ sub _decode_tl {
     if ($len) {
       return if $pos+$len > $end ;
 
-      ($len,$pos) = (unpack("N", "\0" x (4 - $len) . substr($_[0],$pos,$len)), $pos+$len);
+      my $padding = $len < 4 ? "\0" x (4 - $len) : "";
+      ($len,$pos) = (unpack("N", $padding . substr($_[0],$pos,$len)), $pos+$len);
     }
     else {
       unless (exists $larr->{$pos}) {
@@ -682,7 +686,7 @@ sub _scan_indef {
 
     my $tag = substr($_[0], $pos++, 1);
 
-    if((ord($tag) & 0x1f) == 0x1f) {
+    if((unpack("C",$tag) & 0x1f) == 0x1f) {
       my $b;
       do {
 	$tag .= substr($_[0],$pos++,1);
@@ -697,7 +701,8 @@ sub _scan_indef {
       if ($len &= 0x7f) {
 	return if $pos+$len > $end ;
 
-	$pos += $len + unpack("N", "\0" x (4 - $len) . substr($_[0],$pos,$len));
+	my $padding = $len < 4 ? "\0" x (4 - $len) : "";
+	$pos += $len + unpack("N", $padding . substr($_[0],$pos,$len));
       }
       else {
         # reserve another list element
